################################################################################
############################# INNER FUNCTIONS ##################################
################################################################################

plt <- function(dat, main, xlab) {
  col=colorRampPalette(c("darkmagenta", "purple", "dark blue", "blue", "cyan", "green", "yellow", "orange", "red"))(100)
  breaks=seq(0,1,length.out=101)^2
  fields::image.plot(x=dat$bins,z=t(dat$data),zlim=c(0,1), main=main, xlab=xlab,
                     ylab="Error",legend.lab="Density", legend.line=2.2,
                     legend.cex = 0.6, col=col, breaks=breaks)
  par(new=TRUE)
  plot(dat$bins,colMeans(apply(dat$data,2,function(x){x*seq(0,1,length.out=dim(dat$data)[1])}),na.rm=T),
       type="l",ylim=c(0,0.1),col=2,lwd=2,xlim=range(dat$bins),xaxs='i',
       xaxt="n",yaxt="n", xlab ="", ylab = "")
  par(new=TRUE)
  plot(dat$bins,dat$count,
       type="l",ylim=c(0,0.1),col=4,lwd=2,xlim=range(dat$bins),xaxs='i',
       xaxt="n",yaxt="n", xlab ="", ylab = "")
}

################################################################################
############################# SINGLE MODEL RUN #################################
################################################################################
#' XXXXX
#'
#' \code{plt.snow.days.comparison} XXX
#' \code{\link{barplot()}}) XXXX
#'
#' @param data.training The training dataset (a vectdata object, see
#' \code{\link{vectorize.list()}})
#' @param data.model The model dataset (a vectdata object, see
#' \code{\link{vectorize.list()}})
#' @param data.corrected the corrected dataset (a vectdata object, see
#' \code{\link{vectorize.list()}})
#' @param out.plot.dir Path in which the RDS files should be saved. Three files are
#' created: snow_season_training, snow_season_model, snow_season_corrected
#' @param ncores Number of cores to use for parallel computation, using ncores =
#' 1 defaults to serial computing
#' @param swe.threshold The threshold SWE value to define snow days
#'
#' @author Adrien Michel, 2021 (WSL/SLF)
#' @export
plt.error.pattern <- function(data.training, data.model,
                              data.corrected, out.plot.dir, ncores) {

  model <- errorPatternCpp(data1 = data.training$data,
                           data2 = data.model$data,
                           bin_h = 100, bin_v = 20,
                           data.training$dem,
                           data.training$slope,
                           data.training$aspect,
                           data.training$curvature,
                           ncores = ncores)

  corrected <- errorPatternCpp(data1 = data.training$data,
                               data2 = data.corrected$data,
                               bin_h = 100, bin_v = 20,
                               data.training$dem,
                               data.training$slope,
                               data.training$aspect,
                               data.training$curvature,
                               ncores = ncores)

  pdf(paste0(out.plot.dir,"/error_pattern.pdf"),width=12,height=8)

  par(mfrow=c(2,2),mar=c(2.5,4,1.5,1),mgp=c(1.2,0.5,0),oma=c(0,0,0,2))

  plt(model$elevation, main = "Model elevation", xlab = "Elevation (m)")
  plt(corrected$elevation, main = "Corrected elevation", xlab = "Elevation (m)")

  plt(model$aspect, main = "Model aspect", xlab = "aspect (-)")
  plt(corrected$aspect, main = "Corrected aspect", xlab = "Aspect (-)")

  plt(model$slope, main = "Model slope", xlab = "Slope (deg)")
  plt(corrected$slope, main = "Corrected slope", xlab = "Slope (deg)")

  plt(model$curvature, main = "Model curvature", xlab = "Curvature (deg/m)")
  plt(corrected$curvature, main = "Corrected curvature", xlab = "Curvature (deg/m)")

  dev.off()
}
################################################################################
####################### CALIBRATION AND VALIDATION #############################
################################################################################
#' XXXXX
#'
#' \code{plt.snow.days.comparison} XXX
#' \code{\link{barplot()}}) XXXX
#'
#' @param data.training The training dataset (a vectdata object, see
#' \code{\link{vectorize.list()}})
#' @param data.model The model dataset (a vectdata object, see
#' \code{\link{vectorize.list()}})
#' @param data.corrected the corrected dataset (a vectdata object, see
#' \code{\link{vectorize.list()}})
#' @param out.plot.dir Path in which the RDS files should be saved. Three files are
#' created: snow_season_training, snow_season_model, snow_season_corrected
#' @param ncores Number of cores to use for parallel computation, using ncores =
#' 1 defaults to serial computing
#' @param swe.threshold The threshold SWE value to define snow days
#'
#' @author Adrien Michel, 2021 (WSL/SLF)
#' @export
plt.error.pattern.calib <- function(data.training.calib, data.model.calib,
                                    data.corrected.calib, data.training.valid,
                                    data.model.valid, data.corrected.valid,
                                    out.plot.dir, ncores) {

  model.calib <- errorPatternCpp(data1 = data.training.calib$data,
                                 data2 = data.model.calib$data,
                                 bin_h = 100, bin_v = 20,
                                 data.training.calib$dem,
                                 data.training.calib$slope,
                                 data.training.calib$aspect,
                                 data.training.calib$curvature,
                                 ncores = ncores)

  corrected.calib <- errorPatternCpp(data1 = data.training.calib$data,
                                     data2 = data.corrected.calib$data,
                                     bin_h = 100, bin_v = 20,
                                     data.training.calib$dem,
                                     data.training.calib$slope,
                                     data.training.calib$aspect,
                                     data.training.calib$curvature,
                                     ncores = ncores)

  model.valid <- errorPatternCpp(data1 = data.training.valid$data,
                                 data2 = data.model.valid$data,
                                 bin_h = 100, bin_v = 20,
                                 data.training.calib$dem,
                                 data.training.calib$slope,
                                 data.training.calib$aspect,
                                 data.training.calib$curvature,
                                 ncores = ncores)

  corrected.valid <- errorPatternCpp(data1 = data.training.valid$data,
                                     data2 = data.corrected.valid$data,
                                     bin_h = 100, bin_v = 20,
                                     data.training.calib$dem,
                                     data.training.calib$slope,
                                     data.training.calib$aspect,
                                     data.training.calib$curvature,
                                     ncores = ncores)

  pdf(paste0(out.plot.dir,"/error_pattern.pdf"),width=12,height=8)

  par(mfrow=c(4,4),mar=c(2.5,4,1.5,1),mgp=c(1.2,0.5,0),oma=c(0,0,0,2))

  plt(model.calib$elevation, main = "Model calib elevation", xlab = "Elevation (m)")
  plt(corrected.calib$elevation, main = "Corrected calib elevation", xlab = "Elevation (m)")
  plt(model.valid$elevation, main = "Model valid elevation", xlab = "Elevation (m)")
  plt(corrected.valid$elevation, main = "Corrected valid elevation", xlab = "Elevation (m)")

  plt(model.calib$aspect, main = "Model calib aspect", xlab = "aspect (-)")
  plt(corrected.calib$aspect, main = "Corrected calib aspect", xlab = "aspect (-)")
  plt(model.valid$aspect, main = "Model valid aspect", xlab = "aspect (-)")
  plt(corrected.valid$aspect, main = "Corrected valid aspect", xlab = "aspect (-)")

  plt(model.calib$slope, main = "Model calib slope", xlab = "Slope (deg)")
  plt(corrected.calib$slope, main = "Corrected calib slope", xlab = "Slope (deg)")
  plt(model.valid$slope, main = "Model valid slope", xlab = "Slope (deg)")
  plt(corrected.valid$slope, main = "Corrected valid slope", xlab = "Slope (deg)")

  plt(model.calib$curvature, main = "Model calib curvature", xlab = "Curvature (deg/m)")
  plt(corrected.calib$curvature, main = "Corrected calib curvature", xlab = "Curvature (deg/m)")
  plt(model.valid$curvature, main = "Model valid curvature", xlab = "Curvature (deg/m)")
  plt(corrected.valid$curvature, main = "Corrected valid curvature", xlab = "Curvature (deg/m)")

  dev.off()
}