#' Title
#'
#' @param x A
#' @param ... A
#' @export
compute.spatial.cluster <- function(data, params.qm, ncores = 1,
                                  output.file=NULL) {
  if(params.qm$target < 0) {
    stop(paste0("params.qm$target must be > 0'"))
  }
  #Remove useless data
  data$data = NULL
  if(!is.null(output.file)) {
    if(!dir.exists(dirname(output.file))) {
      stop(paste0("Output directory '", dirname(output.file), "' does not exist"))
    } else if(file.access(".", mode=2)==-1) {
      stop(paste0("Output directory '", dirname(output.file), "' seems to be not writable"))
    }
  }

  spatial.cluster <- computeSpatialClusterCpp(elevation = c(data$dem),
                                              slope = c(data$slope),
                                              aspect = c(data$aspect),
                                              curvature = c(data$curvature),
                                              x_coordinates = data$x.coords[data$x.indices],
                                              y_coordinates = data$y.coords[data$y.indices],
                                              target = params.qm$target,
                                              spatial_steps = params.qm$spatial.step,
                                              elevation_steps = params.qm$elevation.step,
                                              slope_steps = params.qm$slope.step,
                                              curvature_steps = params.qm$curvature.step,
                                              aspect_steps = params.qm$aspect.step,
                                              spatial_init = params.qm$spatial.step,
                                              elevation_init = params.qm$elevation.step,
                                              slope_init = params.qm$slope.step,
                                              aspect_init = params.qm$aspect.step,
                                              curvature_init = params.qm$curvature.step,
                                              ncores = ncores)
  if(!is.null(output.file)) {
    saveRDS(spatial.cluster, paste0(output.file))
  }
  return(spatial.cluster)
}
